#!/usr/bin/env python3

import requests

REQ_HEADERS = {
    'Cache-Control': 'no-cache',
}

DEFAULT_HEADER = "X-BZR-REVISION-NUMBER"

SERVICES_INFO = dict(
    CPI=('HEAD', 'https://search.apps.ubuntu.com/api/v1',
         'https://search.apps.staging.ubuntu.com/api/v1'),
    SCA=('GET', 'https://myapps.developer.ubuntu.com/',
         'https://myapps.developer.staging.ubuntu.com/'),
    SSO=('GET', 'https://login.ubuntu.com/',
         'https://login.staging.ubuntu.com/'),
)


def get_deployed_revno(service, environment):
    """Get revno from service."""
    request_type, prod, stag = SERVICES_INFO[service]
    if environment == 'staging':
        url = stag
    else:
        url = prod

    with requests.Session() as session:
        resp = session.get(url)
        if resp.status_code == 200:
            revno = resp.headers[DEFAULT_HEADER]
        else:
            print("{} 500'ed: {}".format(service, resp.text))
            revno = 0

    return service, environment, revno


def main():
    coros = []
    for service in SERVICES_INFO:
        coro = get_deployed_revno(service, 'staging')
        coros.append(coro)
        coro = get_deployed_revno(service, 'production')
        coros.append(coro)

    data = {(service, kind): value for service, kind, value in coros}
    for service in sorted(SERVICES_INFO):
        print("{}:    Staging:    {:>5}".format(service,
                                                data[(service, 'staging')]))
        print("{}:    Production: {:>5}".format(service,
                                                data[(service, 'production')]))


main()
