#
# Copyright (C) 2016 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from olstests import fixtures

from olsstore import (
    config,
    tests,
)
from olsstore.tests import fixtures as store_fixtures


def create_config_from_string(content):
    path = config.Config.save_path()
    with open(path, 'w') as f:
        f.write(content)


class TestConfig(tests.TestCase):

    def setUp(self):
        super().setUp()
        store_fixtures.isolate_for_config(self)

    def test_non_existing_file_succeeds(self):
        conf = config.Config()
        self.assertEqual([], conf.parser.sections())
        self.assertTrue(conf.is_empty())

    def test_existing_file(self):
        existing_conf = config.Config()
        existing_conf.set('foo', 'bar')
        existing_conf.save()
        # Check we find and use the existing conf
        conf = config.Config()
        self.assertEqual('bar', conf.get('foo'))
        self.assertFalse(conf.is_empty())

    def test_irrelevant_sections_are_ignored(self):
        create_config_from_string('''[example.com]\nfoo=bar''')
        conf = config.Config()
        self.assertEqual(None, conf.get('foo'))

    def test_section_from_url(self):
        create_config_from_string('''[example.com]\nfoo=bar''')
        fixtures.override_env(self, 'UBUNTU_SSO_API_ROOT_URL',
                              'http://example.com/api/v2')
        conf = config.Config()
        self.assertEqual('bar', conf.get('foo'))

    def test_save_one_option(self):
        conf = config.Config()
        conf.set('bar', 'baz')
        conf.save()
        new_conf = config.Config()
        self.assertEqual('baz', new_conf.get('bar'))

    def test_clear_preserver_other_sections(self):
        create_config_from_string('''[keep_me]\nfoo=bar\n''')
        conf = config.Config()
        conf.set('bar', 'baz')
        self.assertEqual('baz', conf.get('bar'))
        conf.clear()
        conf.save()
        new_conf = config.Config()
        self.assertEqual(None, new_conf.get('bar'))
        # Picking behind the curtains
        self.assertEqual('bar', new_conf.parser.get('keep_me', 'foo'))
        self.assertTrue(conf.is_empty())


class TestOptions(tests.TestCase):

    def setUp(self):
        super().setUp()
        store_fixtures.isolate_for_config(self)

    def create_config(self, **kwargs):
        conf = config.Config()
        for k, v in kwargs.items():
            conf.set(k, v)
        return conf

    def test_string(self):
        conf = self.create_config(foo='bar')
        self.assertEqual('bar', conf.get('foo'))
