#
# Copyright (C) 2016 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import io
import os
import unittest

import requests

from olstests import (
    fixtures,
    results,
)

from olsstore import tests


class TestTape(unittest.TestCase):

    def setUp(self):
        super().setUp()
        fixtures.set_uniq_cwd(self)

    def test_recording_creates_tape(self):
        path = os.path.join(self.uniq_dir, 'tapes', 'foo')
        self.assertFalse(os.path.exists(path))
        tape = tests.Tape(path, True)
        tape.close()
        self.assertTrue(os.path.exists(path))

    def test_record_utf8_content(self):
        path = os.path.join(self.uniq_dir, 'tapes', 'foo')
        self.assertFalse(os.path.exists(path))
        tape = tests.Tape(path, True)
        response = requests.Response()
        response.__setstate__(dict(_content=b'[]'))
        tape.record(response)
        self.assertEqual(1, len(tape.records))
        record = tape.records[0]
        self.assertIn('content', record)

    def test_record_binary_content(self):
        path = os.path.join(self.uniq_dir, 'tapes', 'foo')
        self.assertFalse(os.path.exists(path))
        tape = tests.Tape(path, True)
        response = requests.Response()
        response.__setstate__(dict(_content=b'\xff'))
        tape.record(response)
        self.assertEqual(1, len(tape.records))
        record = tape.records[0]
        self.assertNotIn('content', record)
        self.assertIn('b64_content', record)


class TestRecordedTest(unittest.TestCase):

    def test_test_without_tape_is_skipped(self):
        class TestWithoutTape(tests.RecordedTestCase):

            recording = False

            def test_me(self):
                pass
        out = io.StringIO()
        res = results.TextResult(out)
        test = TestWithoutTape('test_me')
        # We want to skip but not because the TEST_USER_PASSWORD is not set
        fixtures.override_env(self, 'TEST_USER_PASSWORD', 'anything')
        test.run(res)
        self.assertTrue(res.wasSuccessful())
        self.assertEqual(1, len(res.skip_reasons.values()))
        reason = list(res.skip_reasons.keys())[0]
        self.assertTrue(reason.startswith('No record available'))
